﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Syncfusion.UI.Xaml.TreeGrid;

namespace WpfApp1
{
    public class MainViewModel : ObservableObject
    {
        public MainViewModel()
        {
            InitRootsCommand = new RelayCommand(LoadRootsCommandExecute);
            LoadOnDemandCommand = new AsyncRelayCommand<TreeNode>(LoadOnDemandCommandExecute);

            Roots.Add(new Item { Name = "Manual added" });
        }

        public ObservableCollection<Item> Roots { get; } = new ObservableCollection<Item>();

        public IRelayCommand InitRootsCommand { get; }

        public IAsyncRelayCommand<TreeNode> LoadOnDemandCommand { get; }

        private async Task LoadOnDemandCommandExecute(TreeNode node)
        {
            if (node.Item is Item item)
            {
                var children = await Library.LoadChildrenSimulation(item.Name);



                node.PopulateChildNodes(children);
            }
        }

        private void LoadRootsCommandExecute()
        {
            for (var i = 0; i < 5; i++)
            {
                Roots.Add(new Item { Name = $"Name {i}", });
            }

            // items are in collection, why not show in tree?
        }
    }

    public class Item
    {
        public string Name { get; set; }
    }

    public static class Library
    {
        public static async Task<IEnumerable<Item>> LoadChildrenSimulation(string name)
        {
            var newList = new List<Item>();

            // some delay
            await Task.Delay(2000);


            await Task.Run(() =>
            {
                // async simulation to populate list

                for (var i = 0; i < 5; i++)
                {
                    newList.Add(new Item { Name = $"{name} - Child {i}", });
                }
            });

            return newList;
        }
    }
}
